"""
CopyRight (c) 2024 - Chris Laffra - All Rights Reserved.

Tests for the key-related utility functions in the static.api module.
"""

import sys
import unittest
import unittest.mock

sys.path.append("..")

from tests import mocks # pylint: disable=wrong-import-position,unused-import
from static import api # pylint: disable=wrong-import-position


class TestKeys(unittest.TestCase):
    """
    Tests the key-related utility functions in the static.api module.
    """

    def test_get_key(self):
        """
        Tests the `get_key_from_col_row` function, which generates a spreadsheet-style
        cell reference (e.g. "A1", "AA2", "AB1") from a given column and row numbers.
        """
        self.assertEqual(api.get_key_from_col_row(1, 1), "A1")
        self.assertEqual(api.get_key_from_col_row(27, 2), "AA2")
        self.assertEqual(api.get_key_from_col_row(28, 1), "AB1")

    def test_get_col_row(self):
        """
        Tests the `get_col_row_from_key` function, which extracts the column and row
        numbers from a spreadsheet-style cell reference (e.g. "A1", "AA2", "AB1").
        """
        self.assertEqual(api.get_col_row_from_key("A1"), (1, 1))
        self.assertEqual(api.get_col_row_from_key("AA1"), (27, 1))
        self.assertEqual(api.get_col_row_from_key("AB2"), (28, 2))

    def test_col_name(self):
        """
        Tests the `get_column_name` function, which generates a spreadsheet-style column
        name (e.g. "A", "B", "AA", "AB") from a given column number.
        """
        self.assertEqual(api.get_column_name(1), "A")
        self.assertEqual(api.get_column_name(2), "B")
        self.assertEqual(api.get_column_name(27), "AA")
        self.assertEqual(api.get_column_name(28), "AB")



"""
Copyright (c) 2024 laffra - All Rights Reserved. 

Retrieves and displays a list of sheets in the application's main view.
"""

import constants
import ltk
import state
import menu

import storage

def list_sheets():
    """
    Retrieves and displays a list of sheets in the application's main view.
    """
    state.clear()
    ltk.find("#main").append(
        ltk.Button("New Sheet", ltk.proxy(lambda event: None)).addClass("new-button temporary"),
        ltk.Card(ltk.Div().css("width", 204).css("height", 188)).addClass("document-card temporary"),
        ltk.Card(ltk.Div().css("width", 204).css("height", 188)).addClass("document-card temporary"),
        ltk.Card(ltk.Div().css("width", 204).css("height", 188)).addClass("document-card temporary"),
        ltk.Card(ltk.Div().css("width", 204).css("height", 188)).addClass("document-card temporary"),
        ltk.Card(ltk.Div().css("width", 204).css("height", 188)).addClass("document-card temporary"),
    )
    ltk.find(".temporary").css("opacity", 0).animate(ltk.to_js({ "opacity": 1 }), 2000)
    storage.list_sheets(show_sheet_list)
    ltk.find("#main").animate(ltk.to_js({ "opacity": 1 }), constants.ANIMATION_DURATION)
    ltk.find("#menu").empty().append(menu.create_menu())
    ltk.find("#title").attr("readonly", "true")


def show_sheet_list(sheets):
    """
    Displays a list of sheets in the application's main view. This function is responsible for creating
    the UI elements that represent each sheet, including a screenshot, name, and a click/keyboard
    event handler to load the sheet.
    """
    state.clear()
    ltk.find("#main").empty()

    def create_card(uid, index, runtime, *items):
        def select_doc(event):
            if event.keyCode == 13:
                load_sheet(uid, runtime)

        return (
            ltk.Card(*items)
                .on("click", ltk.proxy(lambda event=None: load_sheet(uid, runtime)))
                .on("keydown", ltk.proxy(select_doc))
                .attr("tabindex", 1000 + index)
                .addClass("document-card")
        )

    ltk.find("#main").append(
        ltk.Container(
            *[
                create_card(
                    sheet.uid,
                    index,
                    "mpy",
                    "",
                    ltk.VBox(
                        ltk.Image(sheet.screenshot),
                        ltk.Text(sheet.name),
                    ),
                )
                for index, sheet in enumerate(sheets)
                if sheet.uid
            ]
        ).prepend(
            ltk.Button("New Sheet", ltk.proxy(lambda event: menu.new_sheet())).addClass(
                "new-button"
            )
        ).css("overflow", "auto").css("height", "100%")
    )
    ltk.find(".document-card").eq(0).focus()
    state.show_message("Select a sheet below or create a new one...")


def load_sheet(uid, runtime):
    """
    Loads a sheet.
    
    Args:
        uid (str): The unique identifier of the document to load.
        runtime (str): The runtime environment to use for the document.
    
    Returns:
        None
    """
    url = f"/?{constants.SHEET_ID}={uid}&{constants.PYTHON_RUNTIME}={runtime}"
    ltk.window.location = url



"""
CopyRight (c) 2024 - Chris Laffra - All Rights Reserved.

This module contains unit tests for cell edits in the `models` module.
"""

import sys
import unittest

sys.path.append("..")

from tests import mocks # pylint: disable=wrong-import-position,unused-import
from static import models # pylint: disable=wrong-import-position
from static import history # pylint: disable=wrong-import-position


class TestCellValue(unittest.TestCase):
    """
    Tests the functionality of cell edits in the `models` module.
    """

    def test_add(self):
        """
        Tests the addition of CellValueChanged edits to the history.
        """
        for n in range(105):
            history.add(models.CellValueChanged(f"A{n}", "before", "after"))
        self.assertEqual(len(history.history), 105)

    def test_style_changed(self):
        """
        Tests the functionality of changing the style of a cell in the `models` module.
        
        This test verifies that the `CellStyleChanged` edit is correctly encoded when the style
        of a cell is changed correctly.
        """
        old_style = {"color": "red"}
        new_style = {"color": "blue", "font-size": "12px", "font-family": ""}
        edit = models.CellStyleChanged("A1", old_style, new_style)
        buffer = []
        edit.encode(buffer)
        actual = "".join(buffer)
        expected = '{"_":"d","key":"A1","style":{"color": "blue"}}'
        self.assertEqual(actual, expected)

    def test_script_changed(self):
        """
        Tests the functionality of changing the script of a cell in the `models` module.
        
        This test verifies that the `CellScriptChanged` edit is correctly encoded when the script
        of a cell is changed correctly.
        """
        old_script = "=hello()"
        new_script = "=world()"
        edit = models.CellScriptChanged("A1", old_script, new_script)
        buffer = []
        edit.encode(buffer)
        actual = "".join(buffer)
        expected = '{"_":"c","key":"A1","script":"=world()"}'
        self.assertEqual(actual, expected)
